#!/bin/bash

#ensure vars that may not be set via args are unset
unset IN_DIR
unset OUT_DIR
unset WORK_DIR

#get script path
SCRIPT_PATH="`dirname \"$0\"`"
SCRIPT_PATH="`( cd \"$SCRIPT_PATH\" && pwd )`"
LOG_FILE="$SCRIPT_PATH/xci2nsp.log"

# set defaults
KEY_FILE="$SCRIPT_PATH"/keys.dat
CUSTOM_KEY=false
CON_BIN="$SCRIPT_PATH"/4nxci
CUSTOM_CON=false
CLEANUP=workspace
RECURSE=false

usage () {
    echo "usage:  "
    echo "  $0 [options]"
    echo
    echo "  -i <dir>:          Input folder (dir with xci files to convert)"
    echo "  -o <dir>:          Output folder (default: same dir as xci file)"
    echo "  -w <dir>:          Working directory (default: same dir as input )"
    echo "  -k <dir/keys.dat>: keys.dat file to use (default: keys.dat in the same dir as the script)"
    echo "  -n <dir/4nxci>:    4nxci binary to use (default: 4nxci in the same dir as the script"
    echo "  -c <level>:        Cleanup level - options are workspace ([default] temp files), all (workspace + xci files), none"
    echo "  -r:                Recurse subdirectories (default is not to recurse)"
    echo "  -h:                this message"
    echo
    echo "Check the log for more verbose output (xci2nsp.log in the same dir as the script)"
    exit 1
}

process_dir () {
  for i in "$1"/*;do
    if [ -d "$i" ];then
      if [[ $RECURSE == true ]]; then
        DIRNAME=$(basename -- "$i")
        echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") checking subdirectory $DIRNAME" | tee -a "$LOG_FILE"
        process_dir "$i"
      fi
    elif [[ -f "$i" && "$i" == *.xci ]]; then
        convert_file "$i"
    fi
  done
}

convert_file () {
  FILE_NAME=$(basename -- "$1") #get name of xci with no path
  FILE_START=`date +%s`
  echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") -Found $FILE_NAME. Converting..." | tee -a "$LOG_FILE"

  "$CON_BIN" -k "$KEY_FILE" "$1" >> "$LOG_FILE" 2>&1 #send all output only to log as it's quite verbose
  
  NSP_NAME=`find . -name "*.nsp" -mtime -10s` #get name of newly created nsp file
  NSP_NAME=$(basename -- "$NSP_NAME") #remove dir name
  echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") --Created as $NSP_NAME" | tee -a "$LOG_FILE"

  FILE_NAME_BASE="${FILE_NAME%.*}" #get name of xci with no extension
  TEMP_OUT=$(dirname "${1}")
  if [[ $OUT_DIR == $TEMP_OUT ]]; then
    echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") --Moving to $OUT_DIR/$FILE_NAME_BASE.nsp" | tee -a "$LOG_FILE"
    mv "$NSP_NAME" "$OUT_DIR/$FILE_NAME_BASE".nsp
  else
    echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") --Moving to $TEMP_OUT/$FILE_NAME_BASE.nsp" | tee -a "$LOG_FILE"
    mv "$NSP_NAME" "$TEMP_OUT/$FILE_NAME_BASE".nsp 
  fi

  if [[ $CLEANUP == 'all' ]]; then
    echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") --Cleaning up workspace and xci files" | tee -a "$LOG_FILE"
    rm -rf 4nxci_extracted_xci #remove each time to prevent bloat
    rm "$i"
  elif [[ $CLEANUP == 'workspace' ]]; then
    echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") --Cleaning up workspace files" | tee -a "$LOG_FILE"
    rm -rf 4nxci_extracted_xci #remove each time to prevent bloat
  fi

  echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") -Finished converting $FILE_NAME" | tee -a "$LOG_FILE"
  FILE_DURATION=$((`date +%s` - $FILE_START))
  echo "Time to process file: $(($FILE_DURATION/60)) min $(($FILE_DURATION%60)) sec" | tee -a "$LOG_FILE"

}

while getopts "hri:o:k:w:c:" opt; do
    case $opt in
        i)
            IN_DIR=$OPTARG
            ;;
        o)
            OUT_DIR=$OPTARG
            ;;
        w)
            WORK_DIR=$OPTARG
            ;;
        k)
            KEY_FILE=$OPTARG
            CUSTOM_KEY=true
            ;;
        n)
            CON_BIN=$OPTARG
            CUSTOM_CON=true
            ;;
        c)
            CLEANUP=$OPTARG
            ;;
        r)
            RECURSE=true
            ;;
        h)
            usage
            exit
            ;;
    esac
done

#if no IN_DIR then show usage
if [ -z "$IN_DIR" ]
then
   usage
   exit 1
fi

#Set OUT_DIR and WORK_DIR to defaults if not set
if [[ -z $OUT_DIR ]]; then
  OUT_DIR=$IN_DIR
fi

if [[ -z $WORK_DIR ]]; then
  WORK_DIR=$IN_DIR
fi

#ensure all paths are full paths
IN_DIR="`( cd \"$IN_DIR\" && pwd )`"
OUT_DIR="`( cd \"$OUT_DIR\" && pwd )`"
WORK_DIR="`( cd \"$WORK_DIR\" && pwd )`"

#ensure 4nxci and keys.dat are present
if [[ ! -x "$CON_BIN" ]]; then
  if [[ $CUSTOM_CON ]]; then
    echo "The specified 4nxci binary does not exist or is not executable"
  else
    echo "4nxci is not in the same folder as this script or is not executable and no other location was specified"
  fi
  exit 1
fi

if [[ ! -f "$KEY_FILE" ]]; then
  if [[ $CUSTOM_KEY ]]; then
    echo "The specified key file does not exist"
  else
    echo "keys.dat is not in the same folder as this script and no other location was specified"
  fi
  exit 1
fi

echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") Processing directory $IN_DIR" | tee -a "$LOG_FILE"
cd "$WORK_DIR"
OVERALL_START=`date +%s`

echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") Looking for .xci files to convert" | tee -a "$LOG_FILE"
process_dir "$IN_DIR"

echo "$(date -u +"%Y-%m-%dT%H:%M:%SZ") Finished processing $IN_DIR" | tee -a "$LOG_FILE"
OVERALL_DURATION=$((`date +%s` - $OVERALL_START))
echo "Time to process directory $IN_DIR: $(($OVERALL_DURATION/60)) min $(($OVERALL_DURATION%60)) sec" | tee -a "$LOG_FILE"
